# $Id: currency.rb,v 1.6 2006/04/08 12:58:32 ianmacd Exp $
#

require 'uri'
require 'net/http'
require 'finance'

module Finance


  # This module handles currency operations.
  #
  module Currency


    # This exception is raised when a currency's exchange rate cannot be
    # determined. This usually means a non-existent currency was used.
    #
    class RateError < StandardError; end

    attr_accessor :currency


    # Where to obtain currency information.
    # 
    CURRENCY_URL = 'http://uk.finance.yahoo.com/currency/convert'

    DEFAULT_CURRENCY_FIELDS =
      %w[ last high low net bid ask offer close open day_range year_range
	  eps div cap nav price last ].map { |field| field.to_sym }


    # Convert one currency to another. The following example converts US $100
    # into Euros:
    #
    #  require 'finance/currency'
    #
    #  Finance::Currency::convert( 'EUR', 'USD', 100 )
    #
    def Currency.convert( to, from, amount=1 )

      # Allow amount to be passed as a string.
      amount = amount.to_f if amount.kind_of?( String )

      # We always want to convert at least 1 of something.
      amount = 1 if amount == 0 || amount.nil?

      # Return immediately if there's no work to do.
      return amount.to_f if from == to

      yahoo = URI.parse( CURRENCY_URL )
      host = yahoo.host
      path = yahoo.path

      # Transparently deal with an HTTP proxy server.
      Net::HTTP::Proxy( *Finance::proxy_data ).start( host, 80 ) do |http|
	response = http.get( path + "?from=%s&to=%s&submit=Convert" %
			     [ from, to ] )
	body = response.body
	body =~ %r{
	  <td\sclass="yfnc_tabledata1">
	    <a\shref="/q\?s=#{from}#{to}=X">#{from}#{to}=X</a>
	  </td>
	  <td\sclass="yfnc_tabledata1"><b>1</b></td>
	  <td\sclass="yfnc_tabledata1">[^<]+</td>
	  <td\sclass="yfnc_tabledata1">(\d+\.\d+)</td>
	}x

	exchange_rate = $1.to_f	  # Must be a Float for conversion to work

	if exchange_rate == 0
	  raise RateError, "Exchange rate could not be determined."
	end

	exchange_rate * amount
      end
    end

  end

end
