# $Id: aex.rb,v 1.5 2006/04/10 11:07:57 ianmacd Exp $
#

require 'net/http'

module Finance
  class Quote


    # = Obtain quotes from Amsterdam Euronext eXchange (AEX)
    #
    #  require 'finance/quote'
    #  require 'finance/quote/aex'
    #
    #  q = Finance::Quote.new
    #  qd = q.fetch( 'aex', 'phi' )
    #  qd = q.fetch( 'dutch', 'phi' )  # alternative form
    #
    # This module fetches information from the {Amsterdam Euronext eXchange
    # (AEX)}[http://www.aex.nl/]. All Dutch stocks and indices are available.
    # 
    # The following labels may be returned by this module:
    #
    # _name_, _last_, _price_ (same as _last_), _date_, _p_change_, _bid_,
    # _ask_, _offer_ (same as _ask_), _open_, _high_, _low_, _close_,
    # _volume_, _currency_, _method_, _exchange_, _time_, _success_,
    # _error_msg_
    #
    module AEX

      # Register myself.
      #
      Finance::Quote::register_plugin( self, { :aex   => :aex,
					       :dutch => :aex } )

      # URL from where to obtain information.
      #
      AEX_URL = 'http://www.aex.nl/scripts/pop/kb.asp'

      LABELS = [ :last, :time, :volume, :bid, :ask,
		 :high, :low, :open, :close, :p_change ]

      # This method takes an Array of Strings or a variable-length list of
      # Strings, each of which should be a stock ticker symbol for which to
      # retrieve information.
      #
      def aex( *symbols )
	symbols.flatten!

	info = QuoteData.new

	symbols = symbols.dup

	symbols.each do |symbol|

	  url = AEX_URL + '?taal=en&alf=' + symbol

	  aex = URI.parse( url )
	  host = aex.host
	  path = aex.path
	  query = aex.query

	  Net::HTTP::Proxy( *Finance::proxy_data ).start( host, 80 ) do |http|

	    if $DEBUG
	      $stderr.printf( "Getting http://%s%s?%s ...\n",
			      host, path, query )
	    end

	    response = http.get( path + '?' + query )
	    body = response.body

	    fields = body.scan( /<td>\r?\n(\d[^<]*?)\r?\n/i ).flatten.
		     map { |field| field.strip }
	    fields.delete_at( 13 ) # time of prev. close
	    fields.delete_at( 11 ) # time of open
	    fields.delete_at( 9 )  # time of low
	    fields.delete_at( 7 )  # time of high
	    fields.delete_at( 4 )  # time of bid
	    info[symbol] = Hash[ *LABELS.zip( fields ).flatten ]

	    # Ensure we got a correct look-up.
	    unless info[symbol][:last]
	      info[symbol][:success] = false
	      next
	    end

	    # Separate date and time.
	    info[symbol][:date] = info[symbol][:time].dup
	    info[symbol][:date].sub!( /\s\d{2}:\d{2}$/, '' )
	    info[symbol][:time].sub!( /^\d{2}\s\w{3}\s\d{4}\s/, '' )

	    # Convert date from DD/MM/YYYY to MM/DD/YYYY.
	    date = info[symbol][:date].split( /\s/ )
	    info[symbol][:date] = [ date[1], date[0], date[2] ].join( '/' )
	    
	    info[symbol][:success] = true
	    info[symbol][:price] = info[symbol][:last]
	    info[symbol][:offer] = info[symbol][:ask]
	    info[symbol][:exchange] = 'Amsterdam Euronext eXchange'
	    info[symbol][:method] = :aex
	    info[symbol][:name] = symbol
	    info[symbol][:currency] = 'EUR'

	  end
	end

        convert( info, Finance::Currency::DEFAULT_CURRENCY_FIELDS )

	yield info if block_given?

	info

      end

    end
  end
end
